@@ -5,6 +5,7 @@ import pandas as pd
 from builtins import object
 import warnings
 
+
 def _check_api(obj_type):
     from transcriptic import api
     if not api:
@@ -24,6 +25,7 @@ class ProtocolPreview(object):
 
 class _BaseObject(object):
     """Base object which other objects inherit from"""
+
     # TODO: Inherit more stuff from here. Need to ensure web has unified fields for objects
     def __init__(self, obj_type, obj_id, attributes, connection=None):
         # If attributes and connection are explicitly provided, just return and not do any smart parsing
@@ -43,7 +45,7 @@ class _BaseObject(object):
 
     def load_object(self, obj_type, obj_id):
         """Find and match object by name"""
-        #TODO: Remove the try/except statement and properly handle cases where objects are not found
+        # TODO: Remove the try/except statement and properly handle cases where objects are not found
         try:
             objects = getattr(self.connection, obj_type + 's')()
         except:
@@ -62,7 +64,7 @@ class _BaseObject(object):
         elif len(matched_objects) == 1:
             return matched_objects[0]
         else:
-            print ("More than 1 match found. Defaulting to the first match: %s" % (matched_objects[0]))
+            print("More than 1 match found. Defaulting to the first match: %s" % (matched_objects[0]))
             return matched_objects[0]
 
 
@@ -92,6 +94,7 @@ class Project(_BaseObject):
         Transcriptic Connection object associated with this specific object
 
     """
+
     def __init__(self, project_id, attributes=None, connection=None):
         """
         Initialize a Project by providing a project name/id. The attributes and connection parameters are generally
@@ -185,7 +188,8 @@ class Run(_BaseObject):
         Transcriptic Connection object associated with this specific object
 
     """
-    def __init__(self, run_id, attributes=None, connection=None):
+
+    def __init__(self, run_id, attributes=None, connection=None, timeout=30.0):
         """
         Initialize a Run by providing a run name/id. The attributes and connection parameters are generally
         not specified unless one wants to manually initialize the object.
@@ -198,18 +202,41 @@ class Run(_BaseObject):
             Attributes of the run
         connection: Optional[transcriptic.config.Connection]
             Connection context. The default context object will be used unless explicitly provided
+        timeout: Optional[float]
+            Timeout in seconds (defaults to 30.0). This will be used when making API calls to fetch data associated with the run.
         """
         super(Run, self).__init__('run', run_id, attributes, connection)
         self.project_id = self.attributes['project']['id']
+        self.timeout = timeout
+        self._data_ids = pd.DataFrame()
         self._instructions = pd.DataFrame()
+        self._containers = pd.DataFrame()
         self._data = pd.DataFrame()
 
+    @property
+    def data_ids(self):
+        """
+        Find and generate a list of datarefs and data_ids associated with this run.
+
+        Returns
+        -------
+        DataFrame
+            Returns a DataFrame of data ids, with datarefs and data_ids as columns
+
+        """
+        if self._data_ids.empty:
+            data_dict = {instruction.attributes['operation']['dataref']: instruction.attributes['dataset']['id'] for instruction in self.instructions['Instructions'] if 'dataset' in instruction.attributes}
+            if len(data_dict) > 0:
+                self._data_ids = pd.DataFrame(sorted(data_dict.items()))
+                self._data_ids.columns = ["dataref", "data_id"]
+        return self._data_ids
+
     @property
     def instructions(self):
         if self._instructions.empty:
             instruction_list = [Instruction(dict(x, **{'project_id': self.project_id, 'run_id': self.id}),
-                                              connection=self.connection)
-                                  for x in self.attributes["instructions"]]
+                                            connection=self.connection)
+                                for x in self.attributes["instructions"]]
             self._instructions = pd.DataFrame(instruction_list)
             self._instructions.columns = ["Instructions"]
             self._instructions.insert(0, "Name", [inst.name for inst in self._instructions.Instructions])
@@ -217,6 +244,21 @@ class Run(_BaseObject):
             self._instructions.insert(2, "Completed", [inst.completed_at for inst in self._instructions.Instructions])
         return self._instructions
 
+    @property
+    def containers(self):
+        if self._containers.empty:
+            container_list = []
+            for ref in Run(self.id).attributes["refs"]:
+                container_list.append(Container(ref["container"]["id"]))
+            self._containers = pd.DataFrame(container_list)
+            self._containers.columns = ["Containers"]
+            self._containers.insert(0, "Name", [container.name for container in self._containers.Containers])
+            self._containers.insert(1, "container_id", [container.id for container in self._containers.Containers])
+            self._containers.insert(2, "Type", [container.container_type.shortname for container in self._containers.Containers])
+            self._containers.insert(3, "Status", [container.attributes["status"] for container in self._containers.Containers])
+            self._containers.insert(4, "Storage Condition", [container.storage for container in self._containers.Containers])
+        return self._containers
+
     @property
     def data(self):
         """
@@ -229,39 +271,24 @@ class Run(_BaseObject):
 
         """
         if self._data.empty:
-            datasets = self.connection.datasets(project_id=self.project_id, run_id=self.id)
-            data_dict = {k: Dataset(datasets[k]["id"], dict(datasets[k], title=k),
-                                    connection=self.connection)
-                         for k in list(datasets.keys()) if datasets[k]}
-            self._data = pd.DataFrame(sorted(list(data_dict.items()), key=lambda x: x[0]))
-            self._data.columns = ["Name", "Datasets"]
-            self._data.insert(1, "DataType", ([ds.operation for ds in self._data.Datasets]))
+            num_datasets = len(self.data_ids)
+            if num_datasets == 0:
+                print("No datasets were found.")
+            else:
+                print("Attempting to fetch %d datasets..." % num_datasets)
+                try:
+                    datasets = self.connection.datasets(project_id=self.project_id, run_id=self.id, timeout=self.timeout)
+                    data_dict = {k: Dataset(datasets[k]["id"], dict(datasets[k], title=k),
+                                            connection=self.connection)
+                                 for k in list(datasets.keys()) if datasets[k]}
+                    self._data = pd.DataFrame(sorted(list(data_dict.items()), key=lambda x: x[0]))
+                    self._data.columns = ["Name", "Datasets"]
+                    self._data.insert(1, "DataType", ([ds.operation for ds in self._data.Datasets]))
+                except Exception:
+                    print('Operation timed out after %d seconds. Returning data_ids instead of Datasets.\nTo try again, increase value of self.timeout and resubmit request.' % self.timeout)
+                    return self.data_ids
         return self._data
 
-    def monitoring(self, instruction_id, data_type='pressure'):
-        """
-        View monitoring data of a given instruction
-
-        Parameters
-        ----------
-        instruction_id: str
-            Instruction id in string form
-        data_type: str
-            Monitoring data type, defaults to 'pressure'
-
-        Returns
-        -------
-        DataFrame
-            Returns a pandas dataframe of the monitoring data
-        """
-        response = self.connection.monitoring_data(
-            project_id=self.project_id,
-            run_id=self.id,
-            instruction_id=instruction_id,
-            data_type=data_type
-        )
-        return pd.DataFrame(response['results'])
-
     def _repr_html_(self):
         return """<iframe src="%s" frameborder="0" allowtransparency="true" \
         style="height:450px" seamless></iframe>""" % \
@@ -292,6 +319,7 @@ class Dataset(_BaseObject):
         Transcriptic Connection object associated with this specific object
 
     """
+
     def __init__(self, data_id, attributes=None, connection=None):
         """
         Initialize a Dataset by providing a data name/id. The attributes and connection parameters are generally
@@ -410,11 +438,34 @@ class Instruction(object):
             self._warps.insert(2, "Completed", [x["reported_completed_at"] for x in warp_list])
         return self._warps
 
+    def monitoring(self, data_type='pressure', grouping=None):
+        """
+        View monitoring data of a given instruction
+
+        Parameters
+        ----------
+        data_type: Optional[str]
+            Monitoring data type, defaults to 'pressure'
+        grouping: Optional[str]
+            Determines whether the values will be grouped, defaults to None. E.g. "5:ms"
+
+        Returns
+        -------
+        DataFrame
+            Returns a pandas dataframe of the monitoring data
+        """
+        response = self.connection.monitoring_data(
+            instruction_id=self.id,
+            data_type=data_type,
+            grouping=grouping
+        )
+        return pd.DataFrame(response['results'])
+
     def _repr_html_(self):
         return """<iframe src="%s" frameborder="0" allowtransparency="true" \
             style="height:1000px" seamless></iframe>""" % \
-               self.connection.get_route('view_instruction', run_id= self.attributes["run_id"],
-                                         project_id= self.attributes["project_id"], instruction_id=self.id)
+               self.connection.get_route('view_instruction', run_id=self.attributes["run_id"],
+                                         project_id=self.attributes["project_id"], instruction_id=self.id)
 
 
 class Container(_BaseObject):
@@ -445,6 +496,10 @@ class Container(_BaseObject):
     container_type: autoprotocol.container_type.ContainerType
         Autoprotocol ContainerType object with many useful container type
         information and functions.
+    cover: str
+        Cover type of container
+    storage: str
+        Storage condition of container
 
         Example Usage:
 
@@ -478,9 +533,11 @@ class Container(_BaseObject):
         super(Container, self).__init__('container', container_id, attributes, connection)
         # TODO: Unify container "label" with name, add Containers route
         self.id = container_id
+        self.cover = self.attributes["cover"]
         self.name = self.attributes["label"]
+        self.storage = self.attributes["storage_condition"]
         self.well_map = {aliquot["well_idx"]: aliquot["name"]
-                        for aliquot in attributes["aliquots"]}
+                         for aliquot in self.attributes["aliquots"]}
         self.container_type = self._parse_container_type()
         self._aliquots = pd.DataFrame()
 
@@ -502,12 +559,11 @@ class Container(_BaseObject):
         if self._aliquots.empty:
             aliquot_list = self.attributes["aliquots"]
             self._aliquots = pd.DataFrame([dict({'Name': x['name'], 'Id': x['id'],
-                                            'Volume': x['volume_ul']}, **x['properties'])
+                                                 'Volume': x['volume_ul']}, **x['properties'])
                                            for x in aliquot_list])
         return self._aliquots
 
-
-def __repr__(self):
+    def __repr__(self):
         """
         Return a string representation of a Container using the specified name.
         (ex. Container('my_plate'))
